
# Environment variables names:

PERMEDCOE_TMPDIR = "PERMEDCOE_TMPDIR"
PERMEDCOE_PROCESSES = "PERMEDCOE_PROCESSES"
PERMEDCOE_GPUS = "PERMEDCOE_GPUS"
PERMEDCOE_MEMORY = "PERMEDCOE_MEMORY"
PERMEDCOE_MOUNT_POINTS = "PERMEDCOE_MOUNT_POINTS"

# Global variables:

SEPARATOR = "------------------------------------------"
