# Always prefer setuptools over distutils
from setuptools import setup, find_packages
import pathlib

here = pathlib.Path(__file__).parent.resolve()

# Get the long description from the README file
long_description = (here / "README.md").read_text(encoding="utf-8")

setup(
    name="NEW_NAME",
    version="0.0.1",
    description="This package provides the NEW_NAME Building Block",
    long_description_content_type="text/markdown",
    long_description=long_description,
    url="TO_BE_DEFINED",
    author="Author",
    author_email="author@example.com",
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3 :: Only",
        "Development Status :: 1 - Planning",
        "Intended Audience :: Science/Research",
        "Operating System :: Unix",
        "Operating System :: POSIX :: Linux",
        "Topic :: Scientific/Engineering :: Bio-Informatics",
        "Topic :: Scientific/Engineering :: Medical Science Apps.",
    ],
    keywords="BuildingBlock, NEW_NAME",
    package_dir={"": "src"},
    packages=find_packages(where="src"),
    python_requires=">=3.6, <4",
    install_requires=["permedcoe>=0.0.6"],
    extras_require={
        "dev": ["check-manifest"],
    },
    # If there are data files included in your packages that need to be
    # installed, specify them here.
    package_data={
        "NEW_NAME": ["assets/*",
                     "definition.json"],
    },
    entry_points={
        "console_scripts": [
            "NEW_NAME=NEW_NAME.__main__:main",
        ],
    },
    project_urls={
        "Bug Reports": "TO_BE_DEFINED",
        "Source": "TO_BE_DEFINED",
    },
)


